package sample;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sas.hls.client.Session;
import com.sas.hls.client.SessionFactory;
import com.sas.hls.client.exception.ConfigurationException;
import com.sas.hls.client.exception.HttpException;
import com.sas.hls.common.exception.ObjectNotFoundException;
import com.sas.hls.drug.analysis.AnalysisDescriptor;
import com.sas.hls.drug.client.DrugServiceManager;
import com.sas.hls.drug.project.ProjectDescriptor;
import com.sas.hls.organization.Organization;
import com.sas.hls.organization.OrganizationDescriptor;
import com.sas.hls.resource.repository.RepositoryContainer;
import com.sas.hls.resource.repository.RepositoryFile;
import com.sas.hls.resource.repository.RepositoryFolder;
import com.sas.hls.resource.repository.RepositoryResource;
import com.sas.hls.resource.repository.exception.RepositoryResourceNotFoundException;
import com.sas.hls.security.acl.Acl;
import com.sas.hls.security.acl.AclContainer;
import com.sas.hls.security.acl.AclEntry;
import com.sas.hls.security.acl.AclEntry.PermissionValue;
import com.sas.hls.security.acl.AclMembers;
import com.sas.hls.security.acl.AclOwner;
import com.sas.hls.security.acl.exception.AclUpdateException;
import com.sas.hls.security.authentication.exception.AuthenticationException;
import com.sas.hls.security.group.GroupContext;
import com.sas.hls.security.group.GroupDescriptor;
import com.sas.hls.security.group.exception.GroupNotFoundException;
import com.sas.hls.security.principal.exception.PrincipalNotFoundException;
import com.sas.hls.security.user.UserDescriptor;
import com.sas.hls.security.user.exception.UserNotFoundException;

public class SASLsafAclService {

    protected static final Log logger = LogFactory.getLog(SASLsafAclService.class);

    private static final String COMMAND_ARGUMENT_USERID_STRING = "-u";
    private static final String COMMAND_ARGUMENT_PASSWORD_STRING = "-p";
    private static final String COMMAND_ARGUMENT_SERVER_STRING = "-s";
    private static final String COMMAND_ARGUMENT_ADMIN_PERMISSION = "-admin";
    private static final String COMMAND_ARGUMENT_READ_PERMISSION = "-read";
    private static final String COMMAND_ARGUMENT_WRITE_PROP_PERMISSION = "-writeProperties";
    private static final String COMMAND_ARGUMENT_WRITE_CONTENT_PERMISSION = "-writeContent";
    private static final String COMMAND_ARGUMENT_DELETE_PERMISSION = "-delete";
    private static final String COMMAND_ARGUMENT_ACL_MEMBER = "-member";
    private static final String COMMAND_ARGUMENT_ACL_OWNER = "-owner";
    private static final String COMMAND_ARGUMENT_GROUP_CONTEXT_ORG = "-org";
    private static final String COMMAND_ARGUMENT_GROUP_CONTEXT_PROJ = "-proj";
    private static final String COMMAND_ARGUMENT_GROUP_CONTEXT_ANALYSIS = "-analysis";

    private static final int COMMAND_HELP = 2;
    private static final String COMMAND_HELP_STRING = "-help";
    private static final int COMMAND_ACL_GET = 501;
    private static final String COMMAND_ACL_GET_STRING = "-acl_get";
    private static final int COMMAND_ACL_UPDATE_OWNER_ACE = 502;
    private static final String COMMAND_ACL_UPDATE_OWNER_ACE_STRING = "-acl_update_owner_ace";
    private static final int COMMAND_ACL_UPDATE_MEMBERS_ACE = 503;
    private static final String COMMAND_ACL_UPDATE_MEMBERS_ACE_STRING = "-acl_update_members_ace";
    private static final int COMMAND_ACL_UPDATE_ADD_USER_ACE = 504;
    private static final String COMMAND_ACL_UPDATE_ADD_USER_ACE_STRING = "-acl_update_addUser_ace";
    private static final int COMMAND_ACL_UPDATE_ADD_GROUP_ACE = 505;
    private static final String COMMAND_ACL_UPDATE_ADD_GROUP_ACE_STRING = "-acl_update_addGroup_ace";
    private static final int COMMAND_ACL_CHANGE_OWNER = 506;
    private static final String COMMAND_ACL_CHANGE_OWNER_STRING = "-acl_change_owner";
    private static final int COMMAND_DEFAULT_ACL_UPDATE_OWNER_ACE = 507;
    private static final String COMMAND_DEFAULT_ACL_UPDATE_OWNER_ACE_STRING = "-default_acl_update_owner_ace";
    private static final int COMMAND_DEFAULT_ACL_UPDATE_MEMBERS_ACE = 508;
    private static final String COMMAND_DEFAULT_ACL_UPDATE_MEMBERS_ACE_STRING = "-default_acl_update_members_ace";
    private static final int COMMAND_DEFAULT_ACL_CHANGE_OWNER = 509;
    private static final String COMMAND_DEFAULT_ACL_CHANGE_OWNER_STRING = "-default_acl_change_owner";

    private static String groupContextType = null;
    private static String groupContextPath = null;

    private static final String ORG = "org";
    private static final String PROJ = "proj";
    private static final String ANALYSIS = "analysis";

    /**
     * Simple command-line sample program using the SAS Drug Development API.
     *
     * @param argv
     */
    public static void main(String[] argv) {

        URL serverURL = null;
        String serverURLString = null;

        String userid = null;
        char[] password = null;
        Session session = null;

        /* Options common across multiple commands */
        String workspaceOrRepositoryPath = null;
        PermissionValue adminPerm = null;
        PermissionValue readPerm = null;
        PermissionValue writePropPerm = null;
        PermissionValue writeContentPerm = null;
        PermissionValue deletePerm = null;
        String memberPrincipal = null;
        String aclOwner = null;
        int command = -1;

        try {

            // Parsing of the command line arguments
            for (int i = 0; i < argv.length; i++) {
                if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_SERVER_STRING)) {
                    i++;
                    if (i < argv.length) {
                        serverURLString = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_USERID_STRING)) {
                    i++;
                    if (i < argv.length)
                        userid = argv[i];
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_PASSWORD_STRING)) {
                    i++;
                    if (i < argv.length)
                        password = argv[i].toCharArray();
                } else if (argv[i].equalsIgnoreCase(COMMAND_HELP_STRING)) {
                    command = COMMAND_HELP;
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_GROUP_CONTEXT_ORG)) {
                    i++;
                    if (i < argv.length) {
                        groupContextType = ORG;
                        groupContextPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_GROUP_CONTEXT_PROJ)) {
                    i++;
                    if (i < argv.length) {
                        groupContextType = PROJ;
                        groupContextPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_GROUP_CONTEXT_ANALYSIS)) {
                    i++;
                    if (i < argv.length) {
                        groupContextType = ANALYSIS;
                        groupContextPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_ACL_MEMBER)) {
                    i++;
                    if (i < argv.length) {
                        memberPrincipal = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_ACL_OWNER)) {
                    i++;
                    if (i < argv.length) {
                        aclOwner = argv[i].equals("null") ? null : argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_ADMIN_PERMISSION)) {
                    i++;
                    if (i < argv.length) {
                        adminPerm = getPermissionValueFromString(argv[i]);
                        if (adminPerm == null) {
                            displayErrorMessage("Unrecognized parameter for admin permission boolean value: " + argv[i]);
                            System.exit(1);
                        }
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_READ_PERMISSION)) {
                    i++;
                    if (i < argv.length) {
                        readPerm = getPermissionValueFromString(argv[i]);
                        if (readPerm == null) {
                            displayErrorMessage("Unrecognized parameter for read permission boolean value: " + argv[i]);
                            System.exit(1);
                        }
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_WRITE_PROP_PERMISSION)) {
                    i++;
                    if (i < argv.length) {
                        writePropPerm = getPermissionValueFromString(argv[i]);
                        if (writePropPerm == null) {
                            displayErrorMessage("Unrecognized parameter for writeProperties permission boolean value: "
                                    + argv[i]);
                            System.exit(1);
                        }
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_WRITE_CONTENT_PERMISSION)) {
                    i++;
                    if (i < argv.length) {
                        writeContentPerm = getPermissionValueFromString(argv[i]);
                        if (writeContentPerm == null) {
                            displayErrorMessage("Unrecognized parameter for writeProperties permission boolean value: "
                                    + argv[i]);
                            System.exit(1);
                        }
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ARGUMENT_DELETE_PERMISSION)) {
                    i++;
                    if (i < argv.length) {
                        deletePerm = getPermissionValueFromString(argv[i]);
                        if (deletePerm == null) {
                            displayErrorMessage("Unrecognized parameter for delete permission boolean value: "
                                    + argv[i]);
                            System.exit(1);
                        }
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_HELP_STRING)) {
                    command = COMMAND_HELP;
                } else if (argv[i].equalsIgnoreCase(COMMAND_ACL_GET_STRING)) {
                    command = COMMAND_ACL_GET;
                    i++;
                    if (i < argv.length) {
                        workspaceOrRepositoryPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ACL_UPDATE_OWNER_ACE_STRING)) {
                    command = COMMAND_ACL_UPDATE_OWNER_ACE;
                    i++;
                    if (i < argv.length) {
                        workspaceOrRepositoryPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ACL_UPDATE_MEMBERS_ACE_STRING)) {
                    command = COMMAND_ACL_UPDATE_MEMBERS_ACE;
                    i++;
                    if (i < argv.length) {
                        workspaceOrRepositoryPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ACL_UPDATE_ADD_USER_ACE_STRING)) {
                    command = COMMAND_ACL_UPDATE_ADD_USER_ACE;
                    i++;
                    if (i < argv.length) {
                        workspaceOrRepositoryPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ACL_UPDATE_ADD_GROUP_ACE_STRING)) {
                    command = COMMAND_ACL_UPDATE_ADD_GROUP_ACE;
                    i++;
                    if (i < argv.length) {
                        workspaceOrRepositoryPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_DEFAULT_ACL_UPDATE_OWNER_ACE_STRING)) {
                    command = COMMAND_DEFAULT_ACL_UPDATE_OWNER_ACE;
                    i++;
                    if (i < argv.length) {
                        workspaceOrRepositoryPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_DEFAULT_ACL_UPDATE_MEMBERS_ACE_STRING)) {
                    command = COMMAND_DEFAULT_ACL_UPDATE_MEMBERS_ACE;
                    i++;
                    if (i < argv.length) {
                        workspaceOrRepositoryPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_ACL_CHANGE_OWNER_STRING)) {
                    command = COMMAND_ACL_CHANGE_OWNER;
                    i++;
                    if (i < argv.length) {
                        workspaceOrRepositoryPath = argv[i];
                    }
                } else if (argv[i].equalsIgnoreCase(COMMAND_DEFAULT_ACL_CHANGE_OWNER_STRING)) {
                    command = COMMAND_DEFAULT_ACL_CHANGE_OWNER;
                    i++;
                    if (i < argv.length) {
                        workspaceOrRepositoryPath = argv[i];
                    }
                } else {
                    displayErrorMessage("unrecognized parameter: " + argv[i]);
                    showUsage();
                    System.exit(1);
                }
            }

            // if serverURL not passed in arg list then we need to prompt for it.
            if (serverURLString == null) {
                serverURLString = String.valueOf(readInput("Server URL: "));
            }

            // validate the URL value
            try {
                serverURL = new URL(serverURLString);
            } catch (MalformedURLException me) {
                recordError("The server URL is not valid.", me);
                System.exit(1);
            }

            // get credential information if we didn't already get it from arguments
            if (userid == null || userid.length() == 0) {
                userid = String.valueOf(readInput("Username: "));
            }
            if (password == null || String.valueOf(password).length() == 0) {
                password = readInput("Password: ");
            }

            // now create our session

            try {
                // create a new authenticated session with LSAF
                session = SessionFactory.newSession(serverURL, userid, String.valueOf(password));
            } catch (AuthenticationException ex) {
                // wrong uid/password, or account is locked etc...
                recordError(ex.getLocalizedMessage(), ex);
            } catch (HttpException ex) {
                // failure in communicating with server etc...
                recordError("Could not create session: " + ex.getLocalizedMessage(), ex);
            } catch (ConfigurationException e) {
                // probably a mismatch between client and server versions of the API
                recordError("Configuration error: " + e.getLocalizedMessage());
            }

            if (session == null) {
                displayNoSessionErrorMessage();
            } else {
                switch (command) {

                    case COMMAND_ACL_GET:
                        doAclGetCommand(session, workspaceOrRepositoryPath);
                        break;

                    case COMMAND_ACL_UPDATE_OWNER_ACE:
                        doAclUpdateOwnerAceCommand(session, workspaceOrRepositoryPath, adminPerm, readPerm,
                                writePropPerm, writeContentPerm, deletePerm);
                        break;

                    case COMMAND_ACL_UPDATE_MEMBERS_ACE:
                        doAclUpdateMembersAceCommand(session, workspaceOrRepositoryPath, adminPerm, readPerm,
                                writePropPerm, writeContentPerm, deletePerm);
                        break;

                    case COMMAND_ACL_UPDATE_ADD_USER_ACE:
                        doAclUpdateAddUserAceCommand(session, workspaceOrRepositoryPath, memberPrincipal, adminPerm,
                                readPerm, writePropPerm, writeContentPerm, deletePerm);
                        break;

                    case COMMAND_ACL_UPDATE_ADD_GROUP_ACE:
                        doAclUpdateAddGroupAceCommand(session, workspaceOrRepositoryPath, groupContextType,
                                groupContextPath, memberPrincipal, adminPerm, readPerm, writePropPerm,
                                writeContentPerm, deletePerm);
                        break;

                    case COMMAND_DEFAULT_ACL_UPDATE_OWNER_ACE:
                        doDefaultAclUpdateOwnerAceCommand(session, workspaceOrRepositoryPath, adminPerm, readPerm,
                                writePropPerm, writeContentPerm, deletePerm);
                        break;

                    case COMMAND_DEFAULT_ACL_UPDATE_MEMBERS_ACE:
                        doDefaultAclUpdateMembersAceCommand(session, workspaceOrRepositoryPath, adminPerm, readPerm,
                                writePropPerm, writeContentPerm, deletePerm);
                        break;

                    case COMMAND_ACL_CHANGE_OWNER:
                        doAclChangeOwner(session, workspaceOrRepositoryPath, aclOwner);
                        break;

                    case COMMAND_DEFAULT_ACL_CHANGE_OWNER:
                        doDefaultAclChangeOwner(session, workspaceOrRepositoryPath, aclOwner);
                        break;

                    default:
                        showUsage();
                        System.exit(1);
                        break;
                }
            }

        } catch (Throwable t) {
            t.printStackTrace();
            recordError("Unexpected Error:  " + t.getLocalizedMessage()); // not expected
        } finally {
            if (session != null) {
                session.invalidate();// release our connection to the server.
            }
        }
    }

    private static void doAclGetCommand(Session session, String path) {
        Acl acl = null;

        try {
            RepositoryResource res = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService()
                    .getResource(path);
            if (res instanceof RepositoryFile) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAcl(res);
            } else if (res instanceof RepositoryFolder) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryFolder) res);
            } else if (res instanceof RepositoryContainer) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryContainer) res);
            }

            displayAcl(acl);
        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Resource not found: " + e.getLocalizedMessage());
        } catch (PrincipalNotFoundException e) {
            displayErrorMessage("Principal not found: " + e.getLocalizedMessage());
        } catch (ObjectNotFoundException e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        } catch (Exception e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        }
    }

    private static void doAclUpdateOwnerAceCommand(Session session, String path, PermissionValue admin,
            PermissionValue read, PermissionValue writeProperties, PermissionValue writeContent, PermissionValue delete) {
        Acl acl = null;

        try {
            RepositoryResource res = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService()
                    .getResource(path);
            if (res instanceof RepositoryFile) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAcl(res);
            } else if (res instanceof RepositoryFolder) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryFolder) res);
            } else {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryContainer) res);
            }

            AclEntry ownerEntry = null;
            for (AclEntry entry : acl.getEntries()) {
                if (entry.getPrincipal() instanceof AclOwner) {
                    ownerEntry = entry;
                    break;
                }
            }
            if (ownerEntry == null) {
                displayErrorMessage("Owner acl entry was not found.");
                System.exit(1);
                return;
            }
            if (admin != null) {
                ownerEntry.setAdminPermission(admin);
            }
            if (read != null) {
                ownerEntry.setReadPermission(read);
            }
            if (writeProperties != null) {
                ownerEntry.setWritePropertiesPermission(writeProperties);
            }
            if (writeContent != null) {
                ownerEntry.setWriteContentPermission(writeContent);
            }
            if (delete != null) {
                ownerEntry.setDeletePermission(delete);
            }

            if (acl instanceof AclContainer) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAclContainer(
                        res instanceof RepositoryFolder ? (RepositoryFolder) res : (RepositoryContainer) res,
                        (AclContainer) acl);
            } else {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAcl(res, acl);
            }

            displaySuccessMessage();
        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Resource not found: " + e.getLocalizedMessage());
        } catch (PrincipalNotFoundException e) {
            displayErrorMessage("Principal not found: " + e.getLocalizedMessage());
        } catch (ObjectNotFoundException e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        } catch (AclUpdateException e) {
            displayErrorMessage("Ace could not be updated. " + e.getLocalizedMessage());
        } catch (Exception e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        }

    }

    private static void doDefaultAclUpdateOwnerAceCommand(Session session, String path, PermissionValue admin,
            PermissionValue read, PermissionValue writeProperties, PermissionValue writeContent, PermissionValue delete) {
        AclContainer aclContainer = null;

        try {
            RepositoryResource res = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService()
                    .getResource(path);
            if (res instanceof RepositoryFile) {
                displayErrorMessage("Path is to a file. Path must be to a container.");
                System.exit(1);
                return;
            } else if (res instanceof RepositoryFolder) {
                aclContainer = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryFolder) res);
            } else {
                aclContainer = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryContainer) res);
            }

            AclEntry ownerEntry = null;
            for (AclEntry entry : aclContainer.getDefaultAcl().getEntries()) {
                if (entry.getPrincipal() instanceof AclOwner) {
                    ownerEntry = entry;
                    break;
                }
            }
            if (ownerEntry == null) {
                displayErrorMessage("Owner acl entry was not found.");
                System.exit(1);
                return;
            }
            if (admin != null) {
                ownerEntry.setAdminPermission(admin);
            }
            if (read != null) {
                ownerEntry.setReadPermission(read);
            }
            if (writeProperties != null) {
                ownerEntry.setWritePropertiesPermission(writeProperties);
            }
            if (writeContent != null) {
                ownerEntry.setWriteContentPermission(writeContent);
            }
            if (delete != null) {
                ownerEntry.setDeletePermission(delete);
            }

            aclContainer = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAclContainer(
                    res instanceof RepositoryFolder ? (RepositoryFolder) res : (RepositoryContainer) res, aclContainer);

            displaySuccessMessage();

        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Resource not found: " + e.getLocalizedMessage());
        } catch (PrincipalNotFoundException e) {
            displayErrorMessage("Principal not found: " + e.getLocalizedMessage());
        } catch (ObjectNotFoundException e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        } catch (AclUpdateException e) {
            displayErrorMessage("Ace could not be updated. " + e.getLocalizedMessage());
        } catch (Exception e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        }

    }

    private static void doAclUpdateMembersAceCommand(Session session, String path, PermissionValue admin,
            PermissionValue read, PermissionValue writeProperties, PermissionValue writeContent, PermissionValue delete) {
        Acl acl = null;

        try {
            RepositoryResource res = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService()
                    .getResource(path);
            if (res instanceof RepositoryFile) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAcl(res);
            } else if (res instanceof RepositoryFolder) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryFolder) res);
            } else {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryContainer) res);
            }

            AclEntry memberEntry = null;
            for (AclEntry entry : acl.getEntries()) {
                if (entry.getPrincipal() instanceof AclMembers) {
                    memberEntry = entry;
                    break;
                }
            }
            if (memberEntry == null) {
                displayErrorMessage("Member acl entry was not found.");
                System.exit(1);
                return;
            }
            if (admin != null) {
                memberEntry.setAdminPermission(admin);
            }
            if (read != null) {
                memberEntry.setReadPermission(read);
            }
            if (writeProperties != null) {
                memberEntry.setWritePropertiesPermission(writeProperties);
            }
            if (writeContent != null) {
                memberEntry.setWriteContentPermission(writeContent);
            }
            if (delete != null) {
                memberEntry.setDeletePermission(delete);
            }

            if (acl instanceof AclContainer) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAclContainer(
                        res instanceof RepositoryFolder ? (RepositoryFolder) res : (RepositoryContainer) res,
                        (AclContainer) acl);
            } else {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAcl(res, acl);
            }

            displaySuccessMessage();

        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Resource not found: " + e.getLocalizedMessage());
        } catch (PrincipalNotFoundException e) {
            displayErrorMessage("Principal not found: " + e.getLocalizedMessage());
        } catch (ObjectNotFoundException e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        } catch (AclUpdateException e) {
            displayErrorMessage("Ace could not be updated. " + e.getLocalizedMessage());
        } catch (Exception e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        }

    }

    private static void doDefaultAclUpdateMembersAceCommand(Session session, String path, PermissionValue admin,
            PermissionValue read, PermissionValue writeProperties, PermissionValue writeContent, PermissionValue delete) {
        AclContainer aclContainer = null;

        try {
            RepositoryResource res = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService()
                    .getResource(path);
            if (res instanceof RepositoryFile) {
                displayErrorMessage("Path is to a file. Path must be to a container.");
                System.exit(1);
                return;
            } else if (res instanceof RepositoryFolder) {
                aclContainer = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryFolder) res);
            } else {
                aclContainer = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryContainer) res);
            }

            AclEntry memberEntry = null;
            for (AclEntry entry : aclContainer.getDefaultAcl().getEntries()) {
                if (entry.getPrincipal() instanceof AclMembers) {
                    memberEntry = entry;
                    break;
                }
            }

            if (memberEntry == null) {
                displayErrorMessage("Member acl entry was not found.");
                System.exit(1);
                return;
            }
            if (admin != null) {
                memberEntry.setAdminPermission(admin);
            }
            if (read != null) {
                memberEntry.setReadPermission(read);
            }
            if (writeProperties != null) {
                memberEntry.setWritePropertiesPermission(writeProperties);
            }
            if (writeContent != null) {
                memberEntry.setWriteContentPermission(writeContent);
            }
            if (delete != null) {
                memberEntry.setDeletePermission(delete);
            }

            aclContainer = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAclContainer(
                    res instanceof RepositoryFolder ? (RepositoryFolder) res : (RepositoryContainer) res, aclContainer);

            displaySuccessMessage();

        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Resource not found: " + e.getLocalizedMessage());
        } catch (PrincipalNotFoundException e) {
            displayErrorMessage("Principal not found: " + e.getLocalizedMessage());
        } catch (ObjectNotFoundException e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        } catch (AclUpdateException e) {
            displayErrorMessage("Ace could not be updated. " + e.getLocalizedMessage());
        } catch (Exception e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        }

    }

    private static void doAclUpdateAddUserAceCommand(Session session, String path, String principalId,
            PermissionValue admin, PermissionValue read, PermissionValue writeProperties, PermissionValue writeContent,
            PermissionValue delete) {
        Acl acl = null;
        UserDescriptor userDesc = null;

        try {
            RepositoryResource res = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService()
                    .getResource(path);
            if (res instanceof RepositoryFile) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAcl(res);
            } else if (res instanceof RepositoryFolder) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryFolder) res);
            } else {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryContainer) res);
            }

            userDesc = ((DrugServiceManager) session.getServiceManager()).getUserService().getUserDescriptorByUserId(
                    principalId);
            acl.setAce(userDesc, admin == null ? PermissionValue.FALSE : admin, read == null ? PermissionValue.TRUE
                    : read, writeProperties == null ? PermissionValue.FALSE : writeProperties,
                    writeContent == null ? PermissionValue.FALSE : writeContent, delete == null ? PermissionValue.FALSE
                            : delete);

            if (acl instanceof AclContainer) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAclContainer(
                        res instanceof RepositoryFolder ? (RepositoryFolder) res : (RepositoryContainer) res,
                        (AclContainer) acl);
            } else {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAcl(res, acl);
            }

            displaySuccessMessage();
        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Resource not found: " + e.getLocalizedMessage());
        } catch (PrincipalNotFoundException e) {
            displayErrorMessage("Principal not found: " + e.getLocalizedMessage());
        } catch (ObjectNotFoundException e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        } catch (UserNotFoundException e) {
            displayErrorMessage("User with specified userId can not found: " + e.getLocalizedMessage());
        } catch (AclUpdateException e) {
            displayErrorMessage("Ace could not be added for user " + principalId + ": " + e.getLocalizedMessage());
        } catch (Exception e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        }

    }

    private static void doAclUpdateAddGroupAceCommand(Session session, String path, String groupContextType,
            String groupContextPath, String principalName, PermissionValue admin, PermissionValue read,
            PermissionValue writeProperties, PermissionValue writeContent, PermissionValue delete) {
        Acl acl = null;
        GroupDescriptor groupDesc = null;
        GroupContext groupC = null;

        try {
            RepositoryResource res = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService()
                    .getResource(path);

            if (res instanceof RepositoryFile) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAcl(res);
            } else if (res instanceof RepositoryFolder) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryFolder) res);
            } else {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryContainer) res);
                if (res.getObjectType().equals(OrganizationDescriptor.OBJECT_TYPE)) {
                    groupC = ((DrugServiceManager) session.getServiceManager()).getOrganizationService()
                            .getOrganizationDescriptorByIdentifier(res.getIdentifier());
                } else if (res.getObjectType().equals(ProjectDescriptor.OBJECT_TYPE)) {
                    groupC = ((DrugServiceManager) session.getServiceManager()).getProjectService()
                            .getProjectDescriptorByIdentifier(res.getIdentifier());
                } else if (res.getObjectType().equals(AnalysisDescriptor.OBJECT_TYPE)) {
                    groupC = ((DrugServiceManager) session.getServiceManager()).getAnalysisService()
                            .getAnalysisDescriptorByIdentifier(res.getIdentifier());
                }
            }

            if (groupC == null) {
                Organization org = ((DrugServiceManager) session.getServiceManager()).getOrganizationService()
                        .getOrganizationByName(getOrgNameFromPath(groupContextPath != null ? groupContextPath : path));
                groupC = org;
                if (groupContextType != null) {
                    if (!groupContextType.equals(ORG)) {
                        ProjectDescriptor projDesc = null;
                        if (groupContextType.equals(PROJ)) {
                            projDesc = ((DrugServiceManager) session.getServiceManager()).getProjectService()
                                    .getProjectDescriptorByName(org, FilenameUtils.getName(groupContextPath));
                            groupC = projDesc;
                        } else if (groupContextType.equals(ANALYSIS)) {
                            projDesc = ((DrugServiceManager) session.getServiceManager()).getProjectService()
                                    .getProjectDescriptorByName(org, getProjNameFromPath(groupContextPath));
                            groupC = ((DrugServiceManager) session.getServiceManager()).getAnalysisService()
                                    .getAnalysisDescriptorByName(projDesc, getAnalysisNameFromPath(groupContextPath));
                        }
                    }
                }
            }

            groupDesc = ((DrugServiceManager) session.getServiceManager()).getGroupService().getGroupByName(groupC,
                    principalName);

            acl.setAce(groupDesc, admin == null ? PermissionValue.FALSE : admin, read == null ? PermissionValue.TRUE
                    : read, writeProperties == null ? PermissionValue.FALSE : writeProperties,
                    writeContent == null ? PermissionValue.FALSE : writeContent, delete == null ? PermissionValue.FALSE
                            : delete);

            if (acl instanceof AclContainer) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAclContainer(
                        res instanceof RepositoryFolder ? (RepositoryFolder) res : (RepositoryContainer) res,
                        (AclContainer) acl);
            } else {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAcl(res, acl);
            }

            displaySuccessMessage();

        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Resource not found: " + e.getLocalizedMessage());
        } catch (PrincipalNotFoundException e) {
            displayErrorMessage("Principal not found: " + e.getLocalizedMessage());
        } catch (ObjectNotFoundException e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        } catch (GroupNotFoundException e) {
            displayErrorMessage("Group with specified group name can not found: " + e.getLocalizedMessage());
        } catch (AclUpdateException e) {
            displayErrorMessage("Ace could not be added for group " + principalName + ": " + e.getLocalizedMessage());
        } catch (Exception e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        }

    }

    private static void doAclChangeOwner(Session session, String path, String newOwner) {
        Acl acl = null;

        try {
            RepositoryResource res = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService()
                    .getResource(path);
            if (res instanceof RepositoryFile) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAcl(res);
            } else if (res instanceof RepositoryFolder) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryFolder) res);
            } else {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryContainer) res);
            }

            UserDescriptor userDesc = ((DrugServiceManager) session.getServiceManager()).getUserService()
                    .getUserDescriptorByUserId(newOwner);
            acl.setOwner(userDesc);

            if (acl instanceof AclContainer) {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAclContainer(
                        res instanceof RepositoryFolder ? (RepositoryFolder) res : (RepositoryContainer) res,
                        (AclContainer) acl);
            } else {
                acl = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAcl(res, acl);
            }
            displaySuccessMessage();
        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Resource not found: " + e.getLocalizedMessage());
        } catch (PrincipalNotFoundException e) {
            displayErrorMessage("Principal not found: " + e.getLocalizedMessage());
        } catch (ObjectNotFoundException e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        } catch (UserNotFoundException e) {
            displayErrorMessage("User with specified userId can not found: " + e.getLocalizedMessage());
        } catch (AclUpdateException e) {
            displayErrorMessage("Owner could not be changed: " + e.getLocalizedMessage());
        } catch (Exception e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        }

    }

    private static void doDefaultAclChangeOwner(Session session, String path, String newOwner) {
        AclContainer aclContainer = null;

        try {
            RepositoryResource res = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService()
                    .getResource(path);
            if (res instanceof RepositoryFile) {
                displayErrorMessage("Path is to a file. Path must be to a container.");
                System.exit(1);
                return;
            } else if (res instanceof RepositoryFolder) {
                aclContainer = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryFolder) res);
            } else {
                aclContainer = ((DrugServiceManager) session.getServiceManager()).getAclService().getAclContainer(
                        (RepositoryContainer) res);
            }

            UserDescriptor userDesc = newOwner != null ? ((DrugServiceManager) session.getServiceManager())
                    .getUserService().getUserDescriptorByUserId(newOwner) : null;
            aclContainer.getDefaultAcl().setOwner(userDesc);

            aclContainer = ((DrugServiceManager) session.getServiceManager()).getAclService().updateAclContainer(
                    res instanceof RepositoryFolder ? (RepositoryFolder) res : (RepositoryContainer) res, aclContainer);
            displaySuccessMessage();
        } catch (RepositoryResourceNotFoundException e) {
            displayErrorMessage("Resource not found: " + e.getLocalizedMessage());
        } catch (PrincipalNotFoundException e) {
            displayErrorMessage("Principal not found: " + e.getLocalizedMessage());
        } catch (ObjectNotFoundException e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        } catch (UserNotFoundException e) {
            displayErrorMessage("User with specified userId can not found: " + e.getLocalizedMessage());
        } catch (AclUpdateException e) {
            displayErrorMessage("Owner could not be changed: " + e.getLocalizedMessage());
        } catch (Exception e) {
            displayErrorMessage("Object with specified path can not found: " + e.getLocalizedMessage());
        }

    }

    private static PermissionValue getPermissionValueFromString(String arg) {
        if (arg == null || arg.isEmpty()) {
            return null;
        }
        PermissionValue rVal = null;
        if (arg.equalsIgnoreCase("true")) {
            rVal = PermissionValue.TRUE;
        } else if (arg.equalsIgnoreCase("false")) {
            rVal = PermissionValue.FALSE;
        } else if (arg.equalsIgnoreCase("deny")) {
            rVal = PermissionValue.DENY;
        } else {
            return null;
        }

        return rVal;
    }

    private static void displayAcl(Acl acl) {

        // System.out.println(acl.toString());
        final StringBuilder sb = new StringBuilder();
        sb.append("Acl [\r\n");
        sb.append("Owner: " + acl.getOwner().getPrincipalId() + "\n");
        for (AclEntry ace : acl.getEntries()) {
            sb.append("Entry: " + ace.getPrincipal().getPrincipalName() + " admin: " + ace.getAdminPermission().name()
                    + " read: " + ace.getReadPermission().name() + " writeProp: "
                    + ace.getWritePropertiesPermission().name() + " writeContent: "
                    + ace.getWriteContentPermission().name() + " delete: " + ace.getDeletePermission().name() + "\n");
        }
        if (acl instanceof AclContainer) {
            sb.append("\n");
            if (((AclContainer) acl).getDefaultAcl().getOwner() == null) {
                sb.append("Default Owner: <creator>");
            } else {
                sb.append("Default Owner: " + ((AclContainer) acl).getDefaultAcl().getOwner().getUserId());
            }
            sb.append("\n");
            for (AclEntry ace : ((AclContainer) acl).getDefaultAcl().getEntries()) {
                sb.append("Default Entry: " + ace.getPrincipal().getPrincipalName() + " admin: "
                        + ace.getAdminPermission().name() + " read: " + ace.getReadPermission().name() + " writeProp: "
                        + ace.getWritePropertiesPermission().name() + " writeContent: "
                        + ace.getWriteContentPermission().name() + " delete: " + ace.getDeletePermission().name()
                        + "\n");
            }
        }
        sb.append("]");
        System.out.println(sb.toString());

    }

    private static void displaySuccessMessage() {
        System.out.println("ok");
    }

    private static void displayErrorMessage(String msg) {
        System.err.println("Error:  " + msg);
        System.exit(1);
    }

    private static void displayNoSessionErrorMessage() {
        displayErrorMessage("Session not established.  Cannot execute this command.");
    }

    // the recordBLAH methods display messages AND log information
    private static void recordError(String msg) {
        recordError(msg, null);
    }

    // simple little error reporting method.
    private static void recordError(String msg, Throwable t) {
        displayErrorMessage(msg);
        if (t != null) {
            logger.error(msg, t);
        } else {
            logger.error(msg);
        }
    }

    private static char[] readInput(String prompt) {
        char[] ret = new char[] { '\0' };
        try {
            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
            System.out.print(prompt);
            ret = (in.readLine()).toCharArray();
        } catch (IOException e) {
            System.err.println("Error reading input.");
        }
        return ret;
    }

    private static String getOrgNameFromPath(String path) {
        if (path == null) {
            return null;
        }
        String orgName = path.substring(1);
        if (orgName.indexOf("/") > 0) {
            orgName = orgName.substring(0, orgName.indexOf("/"));
        }
        return orgName;
    }

    private static String getProjNameFromPath(String path) {
        if (path == null) {
            return null;
        }
        String pathNoFirstSlash = path.substring(1);
        String projectName = pathNoFirstSlash.substring(pathNoFirstSlash.indexOf("/") + 1,
                pathNoFirstSlash.lastIndexOf("/"));

        return projectName;
    }

    private static String getAnalysisNameFromPath(String path) {
        if (path == null) {
            return null;
        }
        String analysisName = path.substring(path.lastIndexOf("/") + 1, path.length());
        return analysisName;
    }

    private static void showUsage() {
        System.out
                .println("Usage: SASLsafAclService [-s <server URL>] [-u <username>] [-p <password>] <command> [<command inputs>]");
        System.out.println("where commands are:\n");
        System.out.println("  -help     ");
        System.out.println("     Shows this list of commands in the sample program.");
        System.out.println("                                                         ");
        System.out.println("  -acl_get <repositoryPath>                                       ");
        System.out.println("    Display permission information for the given repository path.          ");
        System.out
                .println("  -acl_update_owner_ace <repositoryPath> -admin <true|false|deny> -read <true|false|deny> -writeProperties <true|false|deny> -writeContent <true|false|deny> -delete <true|false|deny>                                        ");
        System.out
                .println("    Update permission information for the owner principal of the object represented by the given repository path.          ");
        System.out
                .println("  -acl_update_members_ace <repositoryPath> -admin <true|false|deny> -read <true|false|deny> -writeProperties <true|false|deny> -writeContent <true|false|deny> -delete <true|false|deny>                                        ");
        System.out
                .println("    Update permission information for the members principal of the object represented by the given repository path.          ");
        System.out
                .println("  -acl_update_addUser_ace <repositoryPath> -member <userid> -admin <true|false|deny> -read <true|false|deny> -writeProperties <true|false|deny> -writeContent <true|false|deny> -delete <true|false|deny>                                        ");
        System.out
                .println("    Update permission information by adding an ace for the user with the given userId for the object represented by the given repository path.          ");
        System.out
                .println("  -acl_update_addGroup_ace <repositoryPath> [-org|-proj|-analysis <groupContextPath>] -member <groupName> -admin <true|false|deny> -read <true|false|deny> -writeProperties <true|false|deny> -writeContent <true|false|deny> -delete <true|false|deny>                                        ");
        System.out
                .println("    Update permission information by adding an ace for the group from the given context type and context path with the given name for the object represented by the given repository path.          ");
        System.out
                .println("  -acl_change_owner <repositoryPath> -owner <newOwnerUserId>                                      ");
        System.out
                .println("    Change owner for the object represented by the given repository path to the given user.         ");
        System.out
                .println("  -default_acl_update_owner_ace <repositoryPath> -admin <true|false|deny> -read <true|false|deny> -writeProperties <true|false|deny> -writeContent <true|false|deny> -delete <true|false|deny>                                        ");
        System.out
                .println("    Update permission information for the owner principal in the default acl of the object represented by the given repository path.  Path must be to a container.         ");
        System.out
                .println("  -default_acl_update_members_ace <repositoryPath> -admin <true|false|deny> -read <true|false|deny> -writeProperties <true|false|deny> -writeContent <true|false|deny> -delete <true|false|deny>                                        ");
        System.out
                .println("    Update permission information for the members principal in the default acl of the object represented by the given repository path.  Path must be to a container.         ");
        System.out
                .println("  -default_acl_change_owner <repositoryPath> -owner <newOwnerUserId>                                      ");
        System.out
                .println("    Change default owner in the default acl of the object represented by the given repository path to the given user.  Path must be to a container.  Specify null for newOwnerUserId to set owner to creator.      ");
        System.out.println("                                                                  ");
    }

}
