package sample;

/*************************************************************************
 * LICENSE AGREEMENT FOR SAMPLE CODE OR ADDITIONAL FUNCTIONALITY
 * SAS INSTITUTE INC. IS PROVIDING YOU WITH THE COMPUTER SOFTWARE CODE INCLUDED WITH
 * THIS AGREEMENT ("CODE") ON AN "AS IS" BASIS, AND AUTHORIZES YOU TO USE THE CODE SUBJECT
 * TO THE TERMS HEREOF. BY USING THE CODE, YOU AGREE TO THESE TERMS. YOUR USE OF THE CODE
 * IS AT YOUR OWN RISK. SAS INSTITUTE INC. MAKES NO REPRESENTATION OR WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, NONINFRINGEMENT AND TITLE, WITH RESPECT TO THE CODE.
 *
 * The Code is intended to be used solely as an example for use with part of a product ("Software")
 * you currently have licensed from SAS Institute Inc. or one of its subsidiaries or authorized
 * agents ("SAS"). The Code is designed to add functionality to the Software, but has not necessarily
 * been tested. Accordingly, SAS makes no representation or warranty that the Code will operate
 * error-free. SAS is under no obligation to maintain or support the Code.
 *
 * Neither SAS nor its licensors shall be liable to you or any third party for any general, special,
 * direct, indirect, consequential, incidental or other damages whatsoever arising out of or related
 * to your use or inability to use the Code, even if SAS has been advised of the possibility of such
 * damages.
 *
 * Except as otherwise provided above, the Code is governed by the same agreement that governs the
 * Software. If you do not have an existing agreement with SAS governing the Software, you may not
 * use the Code.
 *
 * SAS and all other SAS Institute Inc. product or service names are registered trademarks or
 * trademarks of SAS Institute Inc. in the USA and other countries. (r) indicates USA registration.
 * Other brand and product names are registered trademarks or trademarks of their respective companies.
 *
 * Copyright 2012, SAS Institute Inc., Cary, NC
 *************************************************************************/
import java.net.MalformedURLException;
import java.net.URL;

import com.sas.hls.client.Session;
import com.sas.hls.client.SessionFactory;
import com.sas.hls.client.exception.ConfigurationException;
import com.sas.hls.client.exception.HttpException;
import com.sas.hls.drug.client.DrugServiceManager;
import com.sas.hls.security.authentication.exception.AuthenticationException;

/**
 * A simplified example of how to use the API to check if a given SAS LSAF resource exists.
 */
public class SASLsafDoesResourceExist {

    /**
     * Entry point to the sample application.
     *
     * @param argv
     */
    public static void main(String[] argv) {
        URL serverURL = null;
        String serverURLString = null;
        String userid = null;
        char[] password = null;
        String resourcePath = null;
        String workspaceOrRepository = "";

        for (int i = 0; i < argv.length; i++) {
            if (argv[i].equalsIgnoreCase("-s")) { // what server to talk to
                i++;
                if (i < argv.length)
                    serverURLString = argv[i];
            } else if (argv[i].equalsIgnoreCase("-u")) { // what is my userid
                i++;
                if (i < argv.length)
                    userid = argv[i];
            } else if (argv[i].equalsIgnoreCase("-p")) { // what is my password
                i++;
                if (i < argv.length)
                    password = argv[i].toCharArray();
            } else {
                workspaceOrRepository = argv[i];
                i++;
                if (i < argv.length) {
                    // what is the resource we want to check
                    resourcePath = argv[i];
                }
            }
        }

        if ((serverURLString == null) || (userid == null) || (password == null) || (resourcePath == null)) {
            showErrorAndUsage("Error:  You did not provide all needed parameter values.", true);
        }

        try { // validate URL string to make sure it is well-formed
            serverURL = new URL(serverURLString);
        } catch (MalformedURLException me) {
            showErrorAndUsage("Error:  The server URL provided is not valid.", true);
        }

        Session session = null;
        try {
            try {
                session = SessionFactory.newSession(serverURL, userid, String.valueOf(password));
            } catch (AuthenticationException ex) { // wrong userid/password, or account is locked etc...
                showErrorAndUsage(ex.getClass().getName() + "\n" + ex.getMessage(), false);
                return;
            } catch (HttpException ex) { // failure in communicating with server etc...
                showErrorAndUsage(ex.getClass().getName() + "\n" + ex.getMessage(), false);
                return;
            } catch (ConfigurationException e) {
                // probably a mismatch between client and server versions of the API
                showErrorAndUsage(e.getClass().getName() + "\n" + e.getLocalizedMessage(), false);
                return;
            }

            boolean doesItExist;
            if (workspaceOrRepository.equalsIgnoreCase("repository")) {
                doesItExist = ((DrugServiceManager) session.getServiceManager()).getRepositoryResourceService().exists(
                        resourcePath);
            } else {
                doesItExist = ((DrugServiceManager) session.getServiceManager()).getWorkspaceResourceService().exists(
                        resourcePath);
            }

            if (doesItExist) {
                System.out.println("true");
            } else {
                System.out.println("false");
            }
            System.exit(0);
        } finally {
            if (session != null) {
                session.invalidate(); // release our connection and free server-side resources.
            }
        }
    }

    private static void showErrorAndUsage(String errorMsg, boolean showUsage) {
        System.err.println(errorMsg + "\n");
        if (showUsage) {
            System.out
                    .println("Usage: SASLsafDoesResourceExist -s <server URL> -u <username> -p <password> <workspace|repository> <resourcePath>");
            System.out.println("                ");
            System.out.println("       Returns:");
            System.out.println("             true or false if the specified resource exists.   ");
        }
        System.exit(1);
    }
}
